﻿;*******************************************************************************
;								 IDENTIFICATION									
;								 --------------									
; Nom du fichier	: PCSC_Module.pb
; Produit			    : 	
; Date de création: 08/03/2016
; Version de base	: 1.0														
; Auteur				  : A. SOURDRILLE
;********************************************************************************

DeclareModule PurePCSCReader   
  Interface Func
    Count.i()
    GetReaderName.s(ReaderNum.i = 1)
    Connect.i()
    Disconnect()
    AddCard()
    Listener(watchATR$="")
    SendToReader.s(inAPDU.s,*reqSW1SW2)
    ReaderSelector()
    Destuctor()
  EndInterface
  Declare.i Constructor(id=#PB_Any)
EndDeclareModule

Module PurePCSCReader
  
  EnableExplicit
  
  XIncludeFile  "../inc/PCSC_Module.pbi"
  XIncludeFile  "../ihm/PCSC_Selector.pbf"
  
  Structure sReaderInfo
    name$
    ActReaderState.i 
    LastReaderState.i
  EndStructure
  
  Structure tByteArray
    *dat
    len.i
  EndStructure
  
  Global.i  WindowUseChoiceValidated = #False
  Global.i  WindowUseChoice_Reader = 0
  Global.i  WindowUseChoice_Protocol = 0
  Global.a  Thread_Kill = #False
  Global    *PurePCSC_Buf = AllocateMemory(#MAX_APDU_LENGTH)
  
  Structure s_PrivateVar
    ;Virtual Table functions
    *vTable.Func
    ;Identify the module
    id.i
    ;For Thread management
    tLock.i
    tSemaphore.i
    ;For PCSC Layer
    hName.s
    hProtocol.i
    hPresent.i
    hConnected.i
    hContext.i
    hCard.i
    *hATR
    hSendRequest.S_SCARD_IO_REQUEST
    hRecvRequest.S_SCARD_IO_REQUEST
    List readers.sReaderInfo()    
    
  EndStructure
  
  ; ------------------------------------------------------------------
  ; \brief Internal function to spy event on connected reader
  ; \param    
  ; \return   
  ; \ingroup  
  ; ------------------------------------------------------------------   
  Procedure ProcessReaderState( OldState.i , NewState.i , *this.s_PrivateVar = #Null)            
    
    Define.a   CardInOld, CardInNew
    Define.a   CardInUseOld, CardInUseNew
    Define.a   ReaderEmOld, ReaderEmNew
    Define.a   CardMuteOld, CardMuteNew
    Define.a   CardUnPowerOld, CardUnPowerNew
    Define.a   CardExclusiveOld, CardExclusiveNew
    Define.a   CardIgnore
    
    ;Get Flags values
    CardInOld   =     isBitSet( OldState , #SCARD_STATE_PRESENT )
    CardInNew   =     isBitSet( NewState , #SCARD_STATE_PRESENT )
    CardInUseOld =    isBitSet( OldState , #SCARD_STATE_INUSE )
    CardInUseNew =    isBitSet( NewState , #SCARD_STATE_INUSE )
    ReaderEmOld =     isBitSet( OldState , #SCARD_STATE_EMPTY )
    ReaderEmNew =     isBitSet( NewState , #SCARD_STATE_EMPTY )
    CardMuteOld =     isBitSet( OldState , #SCARD_STATE_MUTE )
    CardMuteNew =     isBitSet( NewState , #SCARD_STATE_MUTE )
    CardUnPowerOld =  isBitSet( OldState , #SCARD_STATE_UNPOWERED )
    CardUnPowerNew =  isBitSet( NewState , #SCARD_STATE_UNPOWERED )
    CardExclusiveOld =isBitSet( OldState , #SCARD_STATE_EXCLUSIVE )
    CardExclusiveNew =isBitSet( NewState , #SCARD_STATE_EXCLUSIVE ) 
    CardIgnore  =     isBitSet( NewState , #SCARD_STATE_IGNORE ) 
    
    If CardInNew And Not CardInOld And Not CardMuteNew And Not CardIgnore And Not CardInUseNew
      PostEvent( #SCard_evt_Inserted,#Null,0,0,*this)
      ;Debug "CardInserted"            
      ProcedureReturn
    EndIf      
    
    ;si il y avait une carte et que maintenant y'en a plus, c'est qu'elle a été retiré
    If CardInOld And Not CardInNew  
      PostEvent(#SCard_evt_Removed,#Null,0)
      ;Debug "CardRemoved"            
      ProcedureReturn
    EndIf
    
    If CardUnPowerNew And Not CardUnPowerOld   
      PostEvent(#SCard_evt_UnPowered,#Null,0)
      ;Debug "CardUnpowered"
      ProcedureReturn
    EndIf    
    
    If Not CardInUseOld And CardInUseNew
      PostEvent(#SCard_evt_Busy,#Null,0)
      ;Debug "CardInUse"            
      ProcedureReturn      
    EndIf
    
    If CardInUseOld And Not CardInUseNew
      PostEvent(#SCard_evt_Ready,0,0)
      ;Debug "CardReady"            
      ProcedureReturn      
    EndIf        
    
    Define bitsChanged.i = NewState ! OldState
    
  EndProcedure
  
  ; ------------------------------------------------------------------
  ; \fn       Procedure.a     twoCharToByte( Car1.a , Car2.a )
  ; \brief
  ; \param    Car1.a
  ; \param    Car2.a
  ; \return   Ascii
  ; \ingroup
  ; ------------------------------------------------------------------  
  Procedure.a     twoCharToByte( Car1.u , Car2.u )
    Protected MSB_B.a = 0
    Protected LSB_B.a = 0
    
    Select Car1
      Case '0' To '9'
        MSB_B = ( (car1 - $30) << 4 ) & $F0
      Case 'a' To 'f'
        MSB_B = ( (car1 - $57) << 4 ) & $F0
      Case 'A' To 'F'      
        MSB_B = ( (car1 - $37) << 4 ) & $F0
    EndSelect
    
    Select Car2
      Case '0' To '9'
        LSB_B = (car2 - $30) & $0F
      Case 'a' To 'f'
        LSB_B = (car2 - $57)  & $0F
      Case 'A' To 'F'      
        LSB_B = (car2 - $37)  & $0F
    EndSelect  
    
    ProcedureReturn  (MSB_B | LSB_B)
    
  EndProcedure   
  
  ; ------------------------------------------------------------------
  ; \fn       Procedure  StrToByteArray( in.s , *out.tByteArray )
  ; \brief  
  ; \param    in
  ; \param    *out.tByteArray
  ; \return   None
  ; \ingroup
  ; ------------------------------------------------------------------  
  Procedure  StrToByteArray( in.s , *out.tByteArray )
    
    Protected.i            iCounter.i = 0
    Protected.i            jCounter.i = 0
    Protected.i            BufLen
    Protected              SaveString$
    
    SaveString$ = ReplaceString( in , " " , "" )
    
    If ( Len(SaveString$) % 2 <> 0 ) 
      SaveString$ = "0" + SaveString$
    ElseIf ( Len(SaveString$) = 0 )
      ProcedureReturn 0
    EndIf 
    
    *out\len = Len(SaveString$) / 2    
    *out\dat = AllocateMemory( *out\len )
    
    While ( iCounter < Len(SaveString$) )
      CompilerIf #PB_Compiler_Unicode  = 1
        PokeA( *out\dat + jCounter,twoCharToByte(PeekU( @SaveString$ + iCounter) , PeekU( @SaveString$ + iCounter + 2 ) ) )
        iCounter + 4
      CompilerElse
        PokeA( *out\dat + jCounter,twoCharToByte(PeekA( @SaveString$ + iCounter) , PeekA( @SaveString$ + iCounter + 1 ) ) )
        iCounter + 2
      CompilerEndIf
      
      jCounter + 1
    Wend  
    
  EndProcedure  
  
  ; ------------------------------------------------------------------
  ; \fn       Procedure.s  ByteArrayToStr( *in.tByteArray )
  ; \brief
  ; \param    *in.tByteArray
  ; \return   String
  ; \ingroup
  ; ------------------------------------------------------------------  
  Procedure.s  ByteArrayToStr( *in.tByteArray )
    
    Protected iCounter.i
    Protected Response$ = ""
    
    For iCounter = 0 To *in\len - 1
      Response$ + RSet( Hex( PeekA( *in\dat + iCounter ) ) , 2 , "0" )
    Next iCounter
    
    ProcedureReturn Response$
    
  EndProcedure 
  
  ; ------------------------------------------------------------------
  ; \brief Internal threaded function to spy events on every readers
  ; \param    
  ; \return   
  ; \ingroup  
  ; ------------------------------------------------------------------    
  Procedure	Watcher( *data )
    
    Define.SCARDCONTEXT	threadContext = 0
    Define.DWORD		threadReturnCode = 0
    Define.DWORD		cbRecvLength
    NewList				tempReaderName.sReaderInfo()
    Define              Index = 0 
    
    If ( #SCARD_S_SUCCESS = SCardEstablishContext( #SCARD_SCOPE_USER , #Null , #Null , @threadContext ) )
      
      ;Effective Get
      If ( #SCARD_S_SUCCESS = SCardListReaders( threadContext , "" , #Null , @cbRecvLength ) )
        
        ;Allocate memory
        Define buf$= Space(cbRecvLength)
        
        ;Get the list of available reader
        SCardListReaders( threadContext , "" , @buf$ , @cbRecvLength )
        
        ;Init reader name
        AddElement(tempReaderName())      
        
        ;Initialisation
        Define Offset = 0 : cbRecvLength - #PB_Sizeof_Car          
        
        ;Get the name
        While ( cbRecvLength > 0 )            
          If ( PeekC(@buf$+Offset) = 0 ) 
            ;Null Char found, New String begins or may be it's finished
            If ( cbRecvLength = 1 ) : Break : EndIf
            ;Init reader name
            AddElement(tempReaderName())           
          Else              
            tempReaderName()\name$ + Chr(PeekC(@buf$+Offset))              
          EndIf        
          Offset + #PB_Sizeof_Car
          cbRecvLength - 1
        Wend
        
      EndIf 	
      
      Dim	readerState.S_SCARD_READERSTATE(ListSize(tempReaderName())-1)
      
      Index = 0
      ForEach tempReaderName()
        ClearStructure( @readerState(Index) , S_SCARD_READERSTATE )
        readerState(Index)\szReader = @tempReaderName()\name$   
        readerState(Index)\pvUserData = #Null
        readerState(Index)\dwCurrentState = #SCARD_STATE_UNAWARE  
        Index + 1
      Next
      
      SCardGetStatusChange(threadContext,0,@readerState(),ListSize(tempReaderName()))
      
      Repeat                
        Index = 0
        ForEach tempReaderName()     
          readerState(Index)\dwCurrentState = readerState(Index)\dwEventState             
          Index + 1
        Next                
        ;Fonction bloquante tant que readerState reste identique 
        threadReturnCode = SCardGetStatusChange(threadContext,#INFITE_TIMEOUT,@readerState(),ListSize(tempReaderName())) 
        
        Index = 0                
        ForEach tempReaderName()                    
          With tempReaderName()                        
            ;Detected event becomes the actual one
            \ActReaderState = readerState(Index)\dwEventState  
            If isBitSet(\ActReaderState,#SCARD_STATE_CHANGED )
              If ( \ActReaderState <> \LastReaderState )                             
                ProcessReaderState( \LastReaderState,\ActReaderState)                            
                ;Mise à jour du déclencheur
                \LastReaderState = \ActReaderState   	  
              EndIf  
            EndIf
          EndWith                    
          Index + 1
        Next                   
      Until Thread_Kill            
      
      
    EndIf
    
  EndProcedure
  
  ; ------------------------------------------------------------------
  ; \brief Internal function to spy event on selected reader
  ; \param    
  ; \return   
  ; \ingroup  
  ; ------------------------------------------------------------------    
  Procedure  WaitForCard(*this.s_PrivateVar,ATRDetected.s=#Null$,TimeOut.DWORD=#INFITE_TIMEOUT) ;Wait for Card Insertion
    
    Define.i    ReturnCode
    Define 		readerState.S_SCARD_READERSTATE
    Define.i    ActReaderState = #SCARD_STATE_UNAWARE
    Define.i    LastReaderState = #SCARD_STATE_UNAWARE      
    
    With *this
      
      ;Init
      ClearStructure( @readerState , S_SCARD_READERSTATE )
      readerState\szReader = @\hName     
      readerState\pvUserData = #Null
      readerState\dwCurrentState = #SCARD_STATE_UNAWARE  
      readerState\dwEventState   = #SCARD_STATE_UNKNOWN  
      
      ;Premier coup pour établir le contexte courant
      ReturnCode = SCardGetStatusChange(\hContext,#Null, @readerState , 1 ) 
      
      readerState\dwCurrentState = readerState\dwEventState
      ActReaderState = readerState\dwEventState
      LastReaderState = ActReaderState
      
      Repeat
        
        ;On déclenche un évènement d'analyse d'état    	  
        If isBitSet(ActReaderState,#SCARD_STATE_PRESENT )
          \hPresent = #True
          If readerState\cbAtr
            CompilerIf  Defined(INC_GET_ATTRIB,#PB_Constant)
              \hATR = AllocateMemory(readerState\cbAtr)
              If \hATR : CopyMemory( @readerState\rgbAtr,\hATR,readerState\cbAtr) : EndIf
            CompilerEndIf
          EndIf
          Break  
        EndIf                        
        
        readerState\dwCurrentState = readerState\dwEventState
        
        ;Fonction bloquante tant que readerState reste identique 
        ReturnCode = SCardGetStatusChange(\hContext, TimeOut, @readerState , 1 ) 
        
        ;Detected event becomes the actual one
        ActReaderState = readerState\dwEventState 
        
        If ( ActReaderState <> LastReaderState )               
          
          ProcessReaderState(LastReaderState,ActReaderState,*this)
          
          ;Mise à jour du déclencheur
          LastReaderState = ActReaderState   	  
        EndIf        
        
        Delay(10)
        
      ForEver
      
    EndWith
    
  EndProcedure   
  
  ; ------------------------------------------------------------------
  ; \brief External function to allocation PCSC object memory
  ; \param    
  ; \return   
  ; \ingroup  
  ; ------------------------------------------------------------------    
  Procedure.i Constructor( id=#PB_Any ) ; Create Object and allocate memory
    
    ;Allocation Memoire
    Protected *this.s_PrivateVar = AllocateMemory(SizeOf(s_PrivateVar))
    
    If (Not *this)
      ProcedureReturn #False
    EndIf
    
    InitializeStructure(*this, s_PrivateVar)
    
    With *this
      \vTable = ?process_vTable
      \tSemaphore = CreateSemaphore(0)
      If (Not \tSemaphore)
        ClearStructure(*this, s_PrivateVar)
        FreeMemory(*this)
        ProcedureReturn #False
      EndIf
      \tLock = CreateMutex()
      If (Not \tLock)
        FreeSemaphore(\tSemaphore)
        ClearStructure(*this, s_PrivateVar)
        FreeMemory(*this)
        ProcedureReturn #False
      EndIf
      
      ;Check PCSC Context
      Protected.i cbRecvLength
      
      ;First Try to retrieve the length of allocated memory
      If ( #SCARD_S_SUCCESS = SCardEstablishContext( #SCARD_SCOPE_USER , #Null , #Null , @\hContext ) )
        
        ;Identify the PCSC Module, if #PB_Any is used as function parameter, id = hContext
        If id = #PB_Any : \id = \hContext : Else : \id = id : EndIf
        
        ;Effective Get
        If ( #SCARD_S_SUCCESS = SCardListReaders( \hContext , "" , #Null , @cbRecvLength ) )
          
          ;Allocate memory
          Define buf$= Space(cbRecvLength)
          
          ;Get the list of available reader
          SCardListReaders( \hContext , "" , @buf$ , @cbRecvLength )
          
          ;Add the first reader
          AddElement( \readers() )           
          
          ;Initialisation
          Define Offset = 0 : cbRecvLength - #PB_Sizeof_Car          
          
          ;Get the name
          While ( cbRecvLength > 0 )            
            If ( PeekC(@buf$+Offset) = 0 ) 
              ;Null Char found, New String begins
              If ( cbRecvLength = 1 ) : Break : EndIf
              AddElement( \readers() )               
            Else              
              \readers()\name$ + Chr(PeekC(@buf$+Offset))              
            EndIf        
            Offset + #PB_Sizeof_Car
            cbRecvLength - 1
          Wend
          
        EndIf  
      Else
        ProcedureReturn #False
      EndIf      
    EndWith
    
    ProcedureReturn *this
    
  EndProcedure
  
  ; ------------------------------------------------------------------
  ; \brief External function to count available readers
  ; \param    
  ; \return   
  ; \ingroup  
  ; ------------------------------------------------------------------     
  Procedure.i Count(*this.s_PrivateVar)
    ProcedureReturn ListSize(*this\readers())
  EndProcedure
  
  ; ------------------------------------------------------------------
  ; \brief External function to connect the card to the reader
  ; \param    
  ; \return   
  ; \ingroup  
  ; ------------------------------------------------------------------     
  Procedure.s GetReaderName(*this.s_PrivateVar,ReaderNum.i=1)    
    
    If SelectElement(*this\readers(),ReaderNum)
      ProcedureReturn *this\readers()\name$
    EndIf	
    
    ProcedureReturn "" 
    
  EndProcedure  
  
  ; ------------------------------------------------------------------
  ; \brief External function to disconnect the card to the reader
  ; \param    
  ; \return   
  ; \ingroup  
  ; ------------------------------------------------------------------  
  Procedure.i Disconnect(*this.s_PrivateVar)
    
    With *this
      
      If \hConnected
        If ( #SCARD_S_SUCCESS = SCardDisconnect( \hCard , #SCARD_UNPOWER_CARD ) )
          \hConnected = #False
          \hCard = #Null
          Debug "Disconnected"
        EndIf
      EndIf
      
    EndWith
    
  EndProcedure	  
  
  ; ------------------------------------------------------------------
  ; \brief External function to connect the card to the reader
  ; \param    
  ; \return   
  ; \ingroup  
  ; ------------------------------------------------------------------    
  Procedure.i Connect(*this.s_PrivateVar,SplashScreen.i=#False)
    
    ;A reader has been seleted ?
    If *this\hName = "" : ProcedureReturn #False : EndIf
    
    ;Waiting for CardInsertion
    WaitForCard(*this)
    
    With *this
      
      If \hConnected : Disconnect(*this) : EndIf
      
      If ( #SCARD_S_SUCCESS = SCardConnect( \hContext,\hName,#SCARD_SHARE_SHARED,#SCARD_PROTOCOL_Tx,@\hCard,@\hProtocol ) )
        \hConnected = #True
        \hSendRequest\dwProtocol = \hProtocol
        \hRecvRequest\dwProtocol = \hProtocol
        \hSendRequest\cbPciLength = SizeOf(S_SCARD_IO_REQUEST)
        \hRecvRequest\cbPciLength = SizeOf(S_SCARD_IO_REQUEST)  
        
        Debug "Connected"
      Else
        \hConnected = #False
      EndIf
      
    EndWith
    
  EndProcedure
  
  ; ------------------------------------------------------------------
  ; \brief External function to add a card to the spy
  ; \param    
  ; \return   
  ; \ingroup  
  ; ------------------------------------------------------------------   
  Procedure.i AddCard(*this.s_PrivateVar)
    
    ;Waiting for CardInsertion
    WaitForCard(*this)
    
  EndProcedure
  
  ; ------------------------------------------------------------------
  ; \brief Internal function to spy events on every readers
  ; \param    
  ; \return   
  ; \ingroup  
  ; ------------------------------------------------------------------    
  Procedure  Listener(*this.s_PrivateVar,MatchATR.s="")
    
    Define *something
    
    CreateThread( @Watcher() , *something )		
    
  EndProcedure
  
  ; ------------------------------------------------------------------
  ; \brief Internal function to send and receive data to selected reader
  ; \param    
  ; \return   
  ; \ingroup  
  ; ------------------------------------------------------------------   
  Procedure.s SendToReader(*this.s_PrivateVar,inAPDU.s,*reqSW1SW2)
    
    Define  in.tByteArray    
    Define  out.tByteArray    
    Define  RecvBuffLen.i = #MAX_APDU_LENGTH
    Define  RStatus.u = PeekU(*reqSW1SW2)
    Define  ReturnedData.s
    
    ;String to an array of byte
    StrToByteArray(inAPDU,@in)
    
    out\dat = *PurePCSC_Buf
    
    ;Send the C-APDU
    With *this
      
      If ( #SCARD_S_SUCCESS = SCardTransmit( \hCard , @\hSendRequest, in\dat , in\len , 0 , out\dat , @RecvBuffLen ) ) 
        
        ;On récupère le StatusWord a la fin de la trame RPADU
        PokeA( *reqSW1SW2 + 1, PeekA( out\dat + RecvBuffLen - 2 ) )
        PokeA( *reqSW1SW2 + 0, PeekA( out\dat + RecvBuffLen - 1 ) )
        
        If RecvBuffLen >= 2
          ;On retire le status
          out\len = RecvBuffLen - 2
          
          ReturnedData = ByteArrayToStr(@out)
          
          If ( RStatus <> PeekU(*reqSW1SW2) )
            PostEvent(#SCard_evt_UnExpectedSW,#Null,0)
          EndIf
        Else
          PostEvent(#SCard_evt_MuteCard,#Null,0)
          
        EndIf
      Else
        PostEvent(#SCard_evt_TransmitError,#Null,0)
      EndIf
      
    EndWith
    
    ProcedureReturn ReturnedData
    
  EndProcedure  
  
  ; ------------------------------------------------------------------
  ; \brief Open form for Reader Selection/Configuration
  ; \param    
  ; \return   
  ; \ingroup  
  ; ------------------------------------------------------------------ 
  Procedure ReaderSelector(*this.s_PrivateVar)
    
    Define    WindowSelectorSate
    
    ;On charge la fenetre en mode invisible
    OpenWindow_Mod_Reader_Selector()
    
    ;On Charge les protocoles
    AddGadgetItem(#Combo_Mod_Protocol,-1,"T=0")
    AddGadgetItem(#Combo_Mod_Protocol,-1,"T=1")    
    
    ;On affiche le premier élement
    SetGadgetState(#Combo_Mod_Protocol,0)
    
    ;On Charge les readers disponibles
    ForEach *this\readers()
      AddGadgetItem( #Combo_Mod_ReaderList,-1,*this\readers()\name$ )
    Next
    
    ;On affiche le premier element
    SetGadgetState(#Combo_Mod_ReaderList,0)
    
    ;On met le focus sur le Combo Readers
    SetActiveGadget(#Combo_Mod_ReaderList)
    
    ;On rend la form Sticky
    StickyWindow(#Window_Mod_Reader_Selector , #True )
    
    ;Add Shortcut Enter and Escape
    AddKeyboardShortcut(#Window_Mod_Reader_Selector,#PB_Shortcut_Return,#PB_Shortcut_Return)
    AddKeyboardShortcut(#Window_Mod_Reader_Selector,#PB_Shortcut_Escape,#PB_Shortcut_Escape)
    
    ;On affiche la fenetre
    HideWindow( #Window_Mod_Reader_Selector , #False )
    
    ;On gère les évènements sur la fenetre
    Repeat   
      ;On test si la fenetre n'a pas été fermé par quelqu'un
      If IsWindow(#Window_Mod_Reader_Selector)
        Define Event = WaitWindowEvent()        
        If Window_Mod_Reader_Selector_Events(Event) = #False          
          CloseWindow(#Window_Mod_Reader_Selector)
          Break 
        Else
          If Event = #PB_Event_Menu
            Select EventMenu()
              Case #PB_Shortcut_Return
                Button_Mod_ReaderValidate_Events(EventType())
              Case #PB_Shortcut_Escape
                Button_Mod_ReaderCancel_Events(EventType())                
            EndSelect
          EndIf
        EndIf
      Else
        ;Window Selector is not opened, so exit
        Break
      EndIf
    ForEver
    
    ;Remove Shortcut Enter and Escape
    ;RemoveKeyboardShortcut(#Window_Mod_Reader_Selector,#PB_Shortcut_Return)
    ;RemoveKeyboardShortcut(#Window_Mod_Reader_Selector,#PB_Shortcut_Escape)
    
    
    ;Choices have been set by pressing Button_OK
    If WindowUseChoiceValidated
      
      ;Set the name of selected reader
      *this\hName = GetReaderName(*this,WindowUseChoice_Reader)
      
      ;Set the protocol (#SCARD_PROTOCOL_T0=1,#SCARD_PROTOCOL_T1=2)
      *this\hProtocol = WindowUseChoice_Protocol + 1
      
      ;Reset the flag
      WindowUseChoiceValidated = #False
      
    Else      
      *this\hName = ""      
    EndIf
    
  EndProcedure
  
  ;-------------------------------------------------------------------
  ;-Events Functions called by PCSC_Reader.pbf
  ;-------------------------------------------------------------------
  ; ------------------------------------------------------------------
  ; \brief Occurs when Button Validate is click
  ; \param    
  ; \return   
  ; \ingroup  
  ; ------------------------------------------------------------------  
  Procedure Button_Mod_ReaderCancel_Events(EventType)
    WindowUseChoiceValidated = #False
    CloseWindow(#Window_Mod_Reader_Selector)
  EndProcedure
  
  ; ------------------------------------------------------------------
  ; \brief Occurs when Button Cancel is click
  ; \param    
  ; \return   
  ; \ingroup  
  ; ------------------------------------------------------------------  
  Procedure Button_Mod_ReaderValidate_Events(EventType)    
    WindowUseChoice_Reader = GetGadgetState( #Combo_Mod_ReaderList )
    WindowUseChoice_Protocol = GetGadgetState( #Combo_Mod_Protocol )
    WindowUseChoiceValidated = #True
    CloseWindow(#Window_Mod_Reader_Selector)
  EndProcedure
  
  ; ------------------------------------------------------------------
  ; \brief External function to Destroy PCSC object
  ; \param    
  ; \return   
  ; \ingroup  
  ; ------------------------------------------------------------------     
  Procedure Destuctor(*this.s_PrivateVar)
    With *this
      ;Release the PCSC context
      Disconnect(*this)				
      SCardReleaseContext(\hContext)				
      FreeSemaphore(\tSemaphore)
      FreeMutex(\tLock)
      ClearStructure(*this, s_PrivateVar)
    EndWith
    FreeMemory(*this)
  EndProcedure	
  
  
  DataSection
    ;Assignation des pointeurs de fonctions Virtual Table
    process_vTable:
    Data.i @Count()
    Data.i @GetReaderName()
    Data.i @Connect()
    Data.i @Disconnect()
    Data.i @AddCard()
    Data.i @Listener()
    Data.i @SendToReader()
    Data.i @ReaderSelector()
    Data.i @Destuctor()
  EndDataSection
  
EndModule

CompilerIf #PB_Compiler_IsMainFile = #True
  
  IncludeFile "..\evt\PCSC_Events.pbi"
  
  Define pcsc1.PurePCSCReader::Func = PurePCSCReader::Constructor(1)
  Define InOutStatus.u
  Define ReaderCount = pcsc1\Count()
  
  pcsc1\ReaderSelector()
  
  pcsc1\Connect()
  
  ;Expected : NORMAL PROCESSING
  InOutStatus = $9000
  
  Define RAPDU$ = pcsc1\SendToReader("00A4040C0CA000000177504B43532D3135",@InOutStatus)
  
  Debug ">> " + "00A4040C0CA000000177504B43532D3135"
  Debug "<< " + RAPDU$
  Debug "== " + Hex(InOutStatus)
  
  pcsc1\Disconnect()
  
  pcsc1\Destuctor()
  
  End
  
  
  
  ;   pcsc1\Listener("123412")
  ;   
  ;   If OpenWindow(0, 0, 0, 220, 100, "Exemple...", #PB_Window_SystemMenu | #PB_Window_ScreenCentered)
  ;     
  ;     StringGadget(1, 10, 30, 200, 30, "")
  ;     
  ;     AddKeyboardShortcut( 0 , #PB_Shortcut_Escape , #PB_Shortcut_Escape )
  ;     
  ;     Repeat
  ;       event = WaitWindowEvent() 
  ;       
  ;       Select event 
  ;           
  ;         Case #PB_Event_Menu
  ;           If EventMenu() = #PB_Shortcut_Escape
  ;             CloseWindow(0)
  ;             Break
  ;           EndIf
  ;         Case #SCard_evt_Inserted
  ;           SetGadgetText(1,"On a inséré une carte")
  ;           
  ;         Case #SCard_evt_Removed
  ;           SetGadgetText(1,"On a retité une carte")
  ;       EndSelect
  ;     ForEver
  ;   EndIf	  
  
CompilerEndIf
; IDE Options = PureBasic 5.50 (Windows - x86)
; CursorPosition = 828
; Folding = ----
; Markers = 299,369
; EnableThread
; CPU = 1
; CompileSourceDirectory
; Debugger = IDE
; EnableUnicode